import _ from 'lodash';
import React, { useEffect } from 'react';
import { useFieldArray, useForm } from 'react-hook-form';
import { Modal } from 'src/components';
import { Input, Select } from 'src/components/Form';
import { useAuth } from 'src/services/auth';
import { User, UserRole, useUsers } from 'src/services/users';
import { appAccessList, initialUserForm } from './consts';
import { UserModalProps } from './types';

export const CurrentUserModal = ({ open, onClose, user }: UserModalProps) => {
  const { editUserById, userModalLoading } = useUsers();
  const { isAdmin } = useAuth();

  const { control, reset, handleSubmit } = useForm<User>({
    defaultValues: initialUserForm,
  });

  const { fields } = useFieldArray({
    control,
    name: 'app_roles',
  });

  useEffect(() => {
    if (user && !_.isEmpty(user)) {
      reset(user);
    }

    return () => {
      reset(initialUserForm);
    };
  }, [user, reset]);

  const handleSave = async () => {
    try {
      if (user) {
        await handleSubmit((data) => editUserById(data))();
      }

      onClose();
    } catch (e: any) {
      // Continue
    }
  };

  const handleKeyPress = (e: any) => {
    if (e.key === 'Enter' || e.key === 'NumpadEnter') {
      handleSave();
    }
  };

  const handleClose = () => {
    onClose();
  };

  return (
    <Modal onClose={handleClose} open={open} onSave={handleSave} isLoading={userModalLoading} useCancelButton>
      <div className="bg-white px-4">
        <div className="space-y-10 divide-y divide-gray-200">
          <div>
            <div>
              <h3 className="text-lg leading-6 font-medium text-gray-900">Profile</h3>
            </div>

            <div className="mt-6 grid grid-cols-1 gap-y-6 gap-x-4 sm:grid-cols-6">
              <div className="sm:col-span-3">
                <Input control={control} name="name" label="Name" onKeyPress={handleKeyPress} />
              </div>

              <div className="sm:col-span-3">
                <Input control={control} name="email" label="Email" type="email" onKeyPress={handleKeyPress} />
              </div>

              {isAdmin && (
                <>
                  <div className="sm:col-span-3">
                    {fields
                      .filter((field) => field.name === 'dashboard')
                      .map((item, index) => (
                        <Select
                          key={item.id}
                          control={control}
                          name={`app_roles.${index}.role`}
                          label="Role"
                          options={[
                            { value: UserRole.Admin, name: 'Admin' },
                            { value: UserRole.User, name: 'User' },
                          ]}
                        />
                      ))}
                  </div>

                  <div className="sm:col-span-3 opacity-40 cursor-default pointer-events-none select-none">
                    {/* <Select control={control} name="status" label="Status" options={['Admin', 'Inactive']} /> */}
                    <label htmlFor="status" className="block text-sm font-medium text-gray-700">
                      Status
                    </label>
                    <div className="mt-1">
                      <select
                        id="status"
                        name="status"
                        className="shadow-sm focus:ring-primary-500 focus:border-primary-500 block w-full sm:text-sm border-gray-300 rounded-md"
                      >
                        <option>Active</option>
                        <option>Inactive</option>
                        <option>Banned</option>
                      </select>
                    </div>
                  </div>
                </>
              )}
            </div>
          </div>

          {isAdmin && (
            <div>
              <div className="mt-8">
                <h3 className="text-lg leading-6 font-medium text-gray-900">App Access</h3>
              </div>

              <div>
                <div className="flow-root mt-6">
                  <ul className="-my-5 divide-y divide-gray-200 ">
                    {fields
                      .filter((field) => field.name !== 'dashboard')
                      .map((item, index) => (
                        <li className="py-4" key={item.name}>
                          <div className="flex items-center space-x-4">
                            <div className="flex-shrink-0 flex-1 flex items-center">
                              <img
                                className="h-10 w-10 rounded-md overflow-hidden"
                                src={_.find(appAccessList, ['name', item.name!])?.image}
                                alt={item.name ?? 'Image'}
                              />
                              <h3 className="ml-4 text-md leading-6 font-medium text-gray-900">
                                {_.find(appAccessList, ['name', item.name!])?.label}
                              </h3>
                            </div>
                            <div>
                              <Select
                                key={item.id}
                                control={control}
                                name={`app_roles.${index}.role`}
                                options={[
                                  { value: UserRole.Admin, name: 'Admin' },
                                  { value: UserRole.User, name: 'User' },
                                ]}
                              />
                            </div>
                          </div>
                        </li>
                      ))}
                  </ul>
                </div>
              </div>
            </div>
          )}
        </div>
      </div>
    </Modal>
  );
};
